"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateCache = void 0;
const events_1 = require("./events");
const shopify_types_1 = require("./shopify-types");
const helpers_1 = require("./helpers");
const deletionCounter = {};
/**
 * invalidateNode - Recursive function that returns an array of node ids that are invalidated
 * @param gatsbyApi - Gatsby Helpers
 * @param pluginOptions - Plugin Options Object
 * @param nodeMap - Map Object of all nodes that haven't been deleted
 * @param id - The root node to invalidate
 *
 * Note: This function is designed to receive a single top-level node on the first pass
 * and then call itself recuresively afterwards for each nested layer of coupled nodes
 */
function invalidateNode(gatsbyApi, pluginOptions, nodeMap, id) {
    const { typePrefix } = pluginOptions;
    const node = nodeMap[id];
    let invalidatedNodeIds = [];
    if (node) {
        invalidatedNodeIds.push(node.id);
        const type = node.internal.type.replace(`${typePrefix}Shopify`, ``);
        const { coupledNodeFields } = shopify_types_1.shopifyTypes[type];
        if (coupledNodeFields) {
            for (const field of coupledNodeFields) {
                for (const coupledNodeId of node[field]) {
                    invalidatedNodeIds = invalidatedNodeIds.concat(invalidateNode(gatsbyApi, pluginOptions, nodeMap, coupledNodeId));
                }
            }
        }
    }
    return invalidatedNodeIds;
}
function reportNodeDeletion(gatsbyApi, node) {
    const type = (0, helpers_1.parseShopifyId)(node.shopifyId)[1];
    deletionCounter[type] = (deletionCounter[type] || 0) + 1;
    const identifier = node.shopifyId
        ? `shopify ID: ${node.shopifyId}`
        : `internal ID: ${node.id}`;
    gatsbyApi.reporter.info(`Deleting ${type} node with ${identifier}`);
}
function reportDeletionSummary(gatsbyApi) {
    gatsbyApi.reporter.info(`===== SHOPIFY NODE DELETION SUMMARY =====`);
    for (const [key, value] of Object.entries(deletionCounter)) {
        gatsbyApi.reporter.info(`${key}: ${value}`);
    }
}
async function updateCache(gatsbyApi, pluginOptions, lastBuildTime) {
    const { typePrefix } = pluginOptions;
    const nodeMap = Object.keys(shopify_types_1.shopifyTypes)
        .map(type => gatsbyApi.getNodesByType(`${typePrefix}Shopify${type}`))
        .reduce((acc, value) => acc.concat(value), [])
        .reduce((acc, value) => {
        return Object.assign(Object.assign({}, acc), { [value.id]: value });
    }, {});
    const { fetchDestroyEventsSince } = (0, events_1.eventsApi)(pluginOptions);
    const destroyEvents = await fetchDestroyEventsSince(lastBuildTime);
    const invalidatedNodeIds = destroyEvents.reduce((acc, value) => {
        const shopifyId = `gid://shopify/${value.subject_type}/${value.subject_id}`;
        const nodeId = (0, helpers_1.createNodeId)(shopifyId, gatsbyApi, pluginOptions);
        return acc.concat(invalidateNode(gatsbyApi, pluginOptions, nodeMap, nodeId));
    }, []);
    for (const node of Object.values(nodeMap)) {
        if (invalidatedNodeIds.includes(node.id)) {
            gatsbyApi.actions.deleteNode(node);
            reportNodeDeletion(gatsbyApi, node);
        }
        else {
            gatsbyApi.actions.touchNode(node);
        }
    }
    if (invalidatedNodeIds.length > 0) {
        reportDeletionSummary(gatsbyApi);
    }
}
exports.updateCache = updateCache;
//# sourceMappingURL=update-cache.js.map